#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Defines Config Template class
"""

from jinja2 import Environment, StrictUndefined
from jinja2.loaders import FileSystemLoader

from cfgtemplater.base_template import BaseTemplate
from cfgtemplater.extensions import ip_filters, ip_tests


class ConfigTemplate(BaseTemplate):
    """Config Template class
    """

    def init(self):
        self.init_split()
        self.init_yaml()
        self.init_metadata()
        self.init_environment()
        self.init_default_extensions()
        self.init_defaults()

    def init_environment(self):
        """Initiliazes default Jinja2 environment
        """
        self.environment = Environment(
            loader=FileSystemLoader(self.directory),
            lstrip_blocks=True,
            trim_blocks=True,
            undefined=StrictUndefined
        )

    def init_default_extensions(self):
        """Load default cfgtemplater extensions
        """
        for module in [ip_filters, ip_tests]:
            self.load_extension(module)

    def init_defaults(self):
        """Initializes a dict of default values based on YAML variables
        """
        self.defaults = {}
        if self.yaml:
            if 'variables' in self.yaml.keys():
                for variable, attributes in self.yaml['variables'].items():
                    if 'default' in attributes:
                        self.defaults[variable] = attributes['default']

    def load_extension(self, module):
        """Load jinja2 extensions from a python module
        """
        for attr in [getattr(module, f) for f in dir(module)]:
            if callable(attr):
                self.environment.filters[attr.__name__] = attr

    def render(self, attributes=None):
        """Renders the template, merging user values with default values
        """
        attributes = attributes or {}
        template = self.environment.from_string(self.content)
        variables = self.defaults.copy()
        variables.update(attributes)
        return template.render(variables)

    def save(self, filename, attributes=None):
        """ Save rendered template in a file
        """
        attributes = attributes or {}
        with open(filename, 'w') as f:
            f.write(self.render(attributes))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Defines Base Template class
"""

import os
import re
import traceback

import yaml


class BaseTemplate:
    """Base Template class
    """

    YAML_SPLIT_RE = re.compile(r"""
                               \A(?:^---[\n\r])
                               (?P<yaml>.*)
                               (?:^---[\n\r])
                               (?P<template>.*)
                               """,
                               re.MULTILINE | re.DOTALL | re.VERBOSE)

    def __init__(self, filepath):
        self.filepath = os.path.abspath(filepath)
        self.filename = os.path.basename(filepath)
        self.directory = os.path.dirname(self.filepath)

        self.header = None
        self.content = None

        self.yaml = None

        self.init()

    def init(self):
        """Can be overriden to manage default behavior in child classes
        """
        self.init_split()
        self.init_yaml()
        self.init_metadata()

    def init_split(self):
        """Split given template in two parts: YAML front matter and JINJA2 part
        """
        try:
            with open(self.filepath, 'r') as f:
                text = f.read()
                m = re.match(BaseTemplate.YAML_SPLIT_RE, text)
                if m:
                    self.header, self.content = m.group(1, 2)
                else:
                    self.header = None
                    self.content = text
        except IOError as err:
            print(type(err), repr(err), traceback.format_exc())
            raise

    def init_yaml(self):
        """Load header as YAML markup
        """
        if self.header:
            try:
                self.yaml = yaml.load(self.header, Loader=yaml.FullLoader)
            except yaml.YAMLError as err:
                print(err)

    def init_metadata(self):
        """Initialize instance variables for each key found in YAML header
        """
        if self.yaml:
            for key, value in self.yaml.items():
                setattr(type(self), key, value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
"""

import ipaddress


def is_ip(addr):
    try:
        ipaddress.ip_interface(addr)
        return True
    except Exception:
        return False


def is_ipv4(addr):
    try:
        ipaddress.IPv4Interface(addr)
        return True
    except Exception:
        return False


def is_ipv6(addr):
    try:
        ipaddress.IPv6Interface(addr)
        return True
    except Exception:
        return False


def is_ip_net(addr):
    try:
        ipaddress.ip_network(addr)
        return True
    except Exception:
        return False


def is_ipv4_net(addr):
    try:
        ipaddress.IPv4Network(addr)
        return True
    except Exception:
        return False


def is_ipv6_net(addr):
    try:
        ipaddress.IPv6Network(addr)
        return True
    except Exception:
        return False


def is_ipv4_host(addr):
    try:
        address = ipaddress.IPv4Interface(addr)
        network = address.network
        return address.ip in network.hosts()
    except Exception:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Defiles entry point for CLI
"""

import argparse
import ast
import sys
from importlib.machinery import SourceFileLoader

import jinja2.exceptions
import yaml

from cfgtemplater.config_template import ConfigTemplate


class LiteralError(Exception):
    pass


def get_parser():
    """Return CLI parser
    """
    description = "A simple YAML/Jinja2 config generator."
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument(
        '-y',
        metavar="YAML",
        dest='yaml',
        help='YAML file',
        action="append"
    )

    parser.add_argument(
        '-p',
        metavar='KEY=VALUE',
        dest='cli',
        help='key/value pair',
        action="append"
    )

    parser.add_argument(
        '-e',
        metavar='FILE',
        dest='extensions',
        help='Jinja2 extensions modules',
        action='append'
    )

    parser.add_argument(
        'filepath',
        metavar='TEMPLATE',
        help='template'
    )
    return parser


def load_module(filepath):
    """Return extension module
    """
    loader = SourceFileLoader('extensions', filepath)
    return loader.load_module()


def main():
    try:
        parser = get_parser()
        args = parser.parse_args()

        template = ConfigTemplate(args.filepath)

        final_variables = template.defaults.copy()

        if args.extensions:
            for extension in args.extensions:
                module = load_module(extension)
                template.load_extension(module)

        if args.yaml:
            for yaml_file in args.yaml:
                with open(yaml_file, 'r') as f:
                    yaml_variables = yaml.load(f.read(),
                                               Loader=yaml.FullLoader)
                    final_variables.update(yaml_variables)

        if args.cli:
            cli_variables = {}
            for k, v in [pair.split("=") for pair in args.cli]:
                try:
                    value = ast.literal_eval(v)
                    cli_variables[k] = value
                except ValueError:
                    raise LiteralError("Malformed Python literal \
                                       '%s' in '%s=%s'" % (k, k, v))

            final_variables.update(cli_variables)

        print(template.render(final_variables))
        sys.exit(0)

    except jinja2.exceptions.UndefinedError as ex:
        print("ERROR: Variable", ex)
        sys.exit(10)
    except jinja2.exceptions.TemplateNotFound as ex:
        print("ERROR: Subtemplate not found:", ex)
        sys.exit(11)
    except jinja2.exceptions.TemplateAssertionError as ex:
        print("ERROR: Missing filter:", ex)
        sys.exit(12)
    except jinja2.exceptions.TemplateSyntaxError as ex:
        print("ERROR: Syntax error:", ex)
        sys.exit(13)
    except LiteralError as ex:
        print("ERROR:", ex)
        sys.exit(14)
    except yaml.YAMLError as ex:
        print("ERROR:", ex)
        sys.exit(15)
    except FileNotFoundError as ex:
        print("ERROR:", ex)
        sys.exit(16)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
"""

import ipaddress
import re


def first_address(addr):
    """Return the first address of a subnet, or self in case of hostaddress
       is given
    """
    ip = ipaddress.ip_interface(addr)
    prefixlen = ip.network.prefixlen
    version = re.sub('Interface', '', ip.__class__.__name__)
    if any([(version == 'IPv4' and prefixlen == 32),
            (version == 'IPv6' and prefixlen == 128)]):
        return str(ip)
    return f"{next(ip.network.hosts())}/{prefixlen}"


def last_address(addr):
    """Return the last address of a subnet, or self in case of hostaddress
       is given
    """
    ip = ipaddress.ip_interface(addr)
    prefixlen = ip.network.prefixlen
    version = re.sub('Interface', '', ip.__class__.__name__)
    if any([(version == 'IPv4' and prefixlen == 32),
            (version == 'IPv6' and prefixlen == 128)]):
        return str(ip)
    return f"{list(ip.network.hosts())[-1]}/{prefixlen}"


def network(addr):
    """Return the network address of an address
    """
    return str(ipaddress.ip_interface(addr).network)


def broadcast(addr):
    """Return the broadcast address of the current network
    """
    ip = ipaddress.ip_interface(addr)
    return f"{ip.network.broadcast_address}/{ip.network.prefixlen}"


def prefixlen(addr):
    """Return the prefix length of an address
    """
    return str(ipaddress.ip_interface(addr).network.prefixlen)


def netmask(addr):
    """Return the netmask of an address
    """
    return str(ipaddress.ip_interface(addr).netmask)


def hostmask(addr):
    """Return the hostmask of an address
    """
    return str(ipaddress.ip_interface(addr).hostmask)


def compress(addr):
    """Return the compressed form of address, removing leading zeroes
    """
    return str(ipaddress.ip_interface(addr).ip.compressed)


def uncompress(addr):
    """Return the uncompressed form of an address, adding leading zeroes
    """
    ip = ipaddress.ip_interface(addr)
    version = re.sub('Interface', '', ip.__class__.__name__)
    if version == 'IPv4':
        return '.'.join(map(lambda x: '%03d' % int(x), str(ip.ip).split('.')))
    if version == 'IPv6':
        return ip.ip.exploded


def octets(addr):
    """Return a list of octets
    """
    ip = ipaddress.IPv4Interface(addr).ip
    return list(map(int, str(ip).split('.')))


def hextets(addr):
    """Return a list of hextets
    """
    ip = ipaddress.IPv6Interface(addr).ip
    return ip.exploded.split(':')


def address(addr):
    """Return an ip adddress
    """
    return str(ipaddress.ip_interface(addr).ip)
